--<< ====================================================================== >>--
-- Class Setup                                                                --
--<< ====================================================================== >>--
Omniscience = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceDB-2.0", "AceConsole-2.0", "AceDebug-2.0", "CandyBar-2.0")

-- embedded libs
local L = AceLibrary("AceLocale-2.2"):new("Omniscience")
local LSM = LibStub("LibSharedMedia-3.0")
local bitband = bit.band
local playerGUID = UnitGUID("player")

Omniscience.spells = {}

function Omniscience:OnInitialize()
	local defaults = {
		positions = {},
		targetonly = false,

		-- title
		titletext = {[1] = "", [2] = ""},
		titlecolors = {[1] = {0,1,1}, [2] = {1,0,1}}, 
		titlesize = 18,
		-- group
		
		enabled = {hostile = true, friendly = true},
		showunder = {hostile = 1, friendly = 2},
		
		-- bar
		barscale = 1,
		barcolor = "group",
		barheight = 15,
		barwidth = 200,
		textsize = 10,
		texture = "Blizzard",
		reverse = {hostile = false, friendly = false},
		growup = {[1] = false, [2] = false},
	}

	self:RegisterDB("OmniscienceDB")
	self:RegisterDefaults('profile', defaults)
	self:RegisterOpts()

	-- init formating tables
	self.colors = {
		["hostile"] = "yellow",
		["friendly"] = "magenta",
	}


	-- create anchors
	self.anchors = {}
	self.titles = {}
	local yoff = 0
	for i=1, 2 do
		local titlecolors = self.db.profile.titlecolors[i]
		yoff = yoff - 50
		self.anchors[i] = self:CreateAnchor(i, titlecolors[1], titlecolors[2], yoff)
		self.titles[i] = self:CreateTitle(self.db.profile.titletext[i], self.anchors[i], titlecolors[1], titlecolors[2])

		self:RegisterCandyBarGroup("Omniscience-"..i)
		self:SetCandyBarGroupPoint("Omniscience-"..i, "TOP", self.anchors[i], "BOTTOM", 0, 0)
		self:SetCandyBarGroupGrowth("Omniscience-"..i, self.db.profile.growup[i]) 
	end
end

function Omniscience:OnEnable()
	self.bars = {}
	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
end

function Omniscience:OnDisable()
	self:UnregisterAllEvents("Omniscience")
	self:KillAllBars()
	self.bars = nil	
end

function Omniscience:RunTest()
	if not self:IsActive() then self:Print("Activate the addon before attempting to run test bars.") return end
	local duration = 5
	for k in pairs(self.db.profile.enabled) do
		local id = "Omniscience-Test"..k
		local icon = "Interface\\Icons\\Spell_Nature_Drowsy"
		local barcolor = self.colors[k]
		
		local anchor = self.db.profile.showunder[k]
		if self.db.profile.titletext[anchor] then self.titles[anchor]:Show() end
		
		self:RegisterCandyBar(id, duration, L["TestBarText"], icon, barcolor, barcolor, barcolor, "yellow")
		self:SetCandyBarTexture(id, LSM:Fetch("statusbar", self.db.profile.texture))
		self:SetCandyBarReversed(id, self.db.profile.reverse[k])
		self:SetCandyBarHeight(id, self.db.profile.barheight)
		self:SetCandyBarWidth(id, self.db.profile.barwidth)
		self:SetCandyBarFontSize(id, self.db.profile.textsize)
		self:SetCandyBarScale(id, self.db.profile.barscale)
		self:SetCandyBarCompletion(id, function(anchor) Omniscience:UpdateTitle(anchor) end, anchor)
		self:RegisterCandyBarWithGroup(id, "Omniscience-"..anchor)
		-- start this bitch!
		self:StartCandyBar(id, true)

		duration = duration + 5
	end
end

function Omniscience:GetDuration(spellID, player)
	if player then
		return self.spells.cc[spellID][2]
	else
		return self.spells.cc[spellID][4]
	end
end

function Omniscience:COMBAT_LOG_EVENT_UNFILTERED(timestamp, eventType, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, prefix1, prefix2, prefix3, suffix1, suffix2, suffix3)
	if not (eventType == "SPELL_AURA_APPLIED" or eventType == "SPELL_AURA_REMOVED" or eventType == "UNIT_DIED") then return end
	local isDestEnemy = (bitband(dstFlags, COMBATLOG_OBJECT_REACTION_HOSTILE) == COMBATLOG_OBJECT_REACTION_HOSTILE)
	local isDestFriend = (bitband(dstFlags, COMBATLOG_OBJECT_REACTION_FRIENDLY) == COMBATLOG_OBJECT_REACTION_FRIENDLY)
	local isDestPlayer = (bitband(dstFlags, COMBATLOG_OBJECT_CONTROL_PLAYER) == COMBATLOG_OBJECT_CONTROL_PLAYER)

	local spellID = prefix1
	if not self.spells.cc[spellID] then return end

	local duration = self:GetDuration(spellID, isDestPlayer)

	if isDestEnemy then group = "hostile" end
	if isDestFriend then group = "friendly" end

	if self.db.profile.targetonly and not dstGUID == UnitGUID("target") then return end

	if eventType == "SPELL_AURA_APPLIED" then
		self:StartBar(dstName, dstGUID, spellID, group, duration)
	end

	if eventType == "SPELL_AURA_REMOVED" then
		self:KillBar(dstName, dstGUID, spellID, group)
		if isDsetEnemy then self:UpdateTitle(self.db.profile.showunder.hostile) end
		if isDestFriend then self:UpdateTitle(self.db.profile.showunder.friendly) end
	end

	if eventType == "UNIT_DIED" then
		if dstGUID == playerGUID then self:KillAllBars() return end
		
		self:KillBars(dstName, dstGUID, spellID, group)
		if isDsetEnemy then self:UpdateTitle(self.db.profile.showunder.hostile) end
		if isDestFriend then self:UpdateTitle(self.db.profile.showunder.friendly) end

	end
end

function Omniscience:StartBar(unitName, unitGUID, spellID, group, duration)
	local id = "Omniscience-"..unitGUID.."-"..spellID.."-"..group
	local spell,_,icon = GetSpellInfo(spellID)
	local barcolor = self.colors[group]
	if string.find(unitName,"-") then unitName = string.gsub(unitName,"-.*","") end
	local text = unitName..": "..spell
	
	local anchor = self.db.profile.showunder[group]
	if self.db.profile.titletext[anchor] then self.titles[anchor]:Show() end

	self:RegisterCandyBar(id, duration, text, icon, barcolor, barcolor, barcolor, "yellow")
	self:SetCandyBarTexture(id, LSM:Fetch("statusbar", self.db.profile.texture))
	self:SetCandyBarReversed(id, self.db.profile.reverse[group])
	self:SetCandyBarHeight(id, self.db.profile.barheight)
	self:SetCandyBarWidth(id, self.db.profile.barwidth)
	self:SetCandyBarFontSize(id, self.db.profile.textsize)
	self:SetCandyBarScale(id, self.db.profile.barscale)
	self:SetCandyBarOnClick(id, function(id,button,unitName,unitGUID,spellID,group) self:OnClick(id,button,unitName,unitGUID,spellID,group) end, unitName,unitGUID,spellID,group)
	self:SetCandyBarCompletion(id, function(unitName,unitGUID,spellID,group) self:BarEnd(unitName,unitGUID,spellID,group) end, unitName,unitGUID,spellID,group)
	self:RegisterCandyBarWithGroup(id, "Omniscience-"..anchor)
	self:StartCandyBar(id, true)
	self.bars[id] = id
end

function Omniscience:BarEnd(unitName,unitGUID,spellID ,group) 
	local spell = GetSpellInfo(spellID)
	local anchor = self.db.profile.showunder[group]
	
	self.bars["Omniscience-"..unitGUID..spellID] = nil
	self:UpdateTitle(anchor)
end

function Omniscience:OnClick(id, button, unitName, unitGUID, spellID, group)
	local spell = GetSpellInfo(spellID)

	if IsShiftKeyDown() and button == "RightButton" then
		self:KillGroup(group)

	elseif button == "RightButton" then
		self:KillBar(nil, nil, nil, nil, id)
		self:UpdateTitles()
	end
end

function Omniscience:KillBar(unitName, unitGUID, spellID, group, id)
	if not id and unitGUID and spellID then id = "Omniscience-"..unitGUID.."-"..spellID.."-"..group end
	if id then
		self:SetCandyBarFade(id, 1)
		self:SetCandyBarColor(id, "yellow")
		self:StopCandyBar(id)
		self.bars[id] = nil
	end
end

function Omniscience:KillBars(unitGUID)
	for i in pairs(self.bars) do
		if string.find(i, unitGUID) then
			self:KillBar(nil, nil, nil, nil,i)
		end
	end
	self:UpdateTitles()
end

function Omniscience:KillAllBars()
	for i in pairs(self.bars) do
		self:KillBar(nil, nil, nil, nil,i)
	end
	self:HideTitles()
end

function Omniscience:KillGroup(group)
	for i in pairs(self.bars) do
		if string.find(i, group) then
			self:KillBar(nil, nil, nil, nil,i)
		end
	end
	self:UpdateTitle(self.db.profile.showunder[group])
end

function Omniscience:ToggleAnchors()
	if self.anchors[1]:IsVisible() then
		for k, v in pairs(self.anchors) do
			v:Hide()
		end
	else
		for k, v in pairs(self.anchors) do
			v:Show()
		end
	end
end

function Omniscience:CreateAnchor(id, cRed, cGreen, cBlue, yoff)
	local f = CreateFrame("Button", nil, UIParent)
	f:SetWidth(100)
	f:SetHeight(25)
	
	f.owner = self
	
	if not self.db.profile.positions[id] then self.db.profile.positions[id] = {} end

	if self.db.profile.positions[id].point and self.db.profile.positions[id].relPoint and self.db.profile.positions[id].x and self.db.profile.positions[id].y then
		f:ClearAllPoints()
		f:SetPoint(self.db.profile.positions[id].point, UIParent, self.db.profile.positions[id].relPoint, self.db.profile.positions[id].x, self.db.profile.positions[id].y)
	else
		f:SetPoint("TOP", UIErrorsFrame, "BOTTOM", 0, yoff)
	end
	
	f:SetScript("OnDragStart", function() this:StartMoving() end )
	f:SetScript("OnDragStop",
		function()
			this:StopMovingOrSizing()
			local point, _, relPoint, x, y = this:GetPoint()
			this.owner.db.profile.positions[id].point = point
			this.owner.db.profile.positions[id].relPoint = relPoint
			this.owner.db.profile.positions[id].x = math.floor(x)
			this.owner.db.profile.positions[id].y = math.floor(y)
        end)

	f:SetBackdrop({bgFile = "Interface/Tooltips/UI-Tooltip-Background",
                                            edgeFile = "Interface/Tooltips/UI-Tooltip-Border",
                                            tile = false, tileSize = 16, edgeSize = 16,
                                            insets = { left = 5, right =5, top = 5, bottom = 5 }})
	f:SetBackdropColor(cRed,cGreen,cBlue,.6)
	f:SetMovable(true)
	f:RegisterForDrag("LeftButton")

	f.Text = f:CreateFontString(nil, "OVERLAY")
	f.Text:SetFontObject(GameFontNormalSmall)
	f.Text:ClearAllPoints()
	f.Text:SetTextColor(1, 1, 1, 1)
	f.Text:SetWidth(100)
	f.Text:SetHeight(25)
	f.Text:SetPoint("TOPLEFT", f, "TOPLEFT")
	f.Text:SetJustifyH("CENTER")
	f.Text:SetJustifyV("MIDDLE")
	f.Text:SetText("Omniscience : "..id)
	
	f:Hide()

	return f
end

function Omniscience:UpdateTitle(anchor)
	if not anchor then return end
	for i in pairs(self.bars) do
		if self:IsCandyBarRegisteredWithGroup(i, "Omniscience-"..anchor) then 
			return 
		end
	end
	self.titles[anchor]:Hide()
end

function Omniscience:UpdateTitles()
	self:UpdateTitle(1)
	self:UpdateTitle(2)
end

function Omniscience:HideTitles()
	self.titles[1]:Hide()
	self.titles[2]:Hide()
end

function Omniscience:CreateTitle(text, pframe, cRed,cGreen,cBlue)
	local f = CreateFrame("Button",nil,UIParent)
	f:EnableMouse(false)
	f:ClearAllPoints()
	f:SetWidth(200)
	f:SetHeight(25)
	f:SetPoint("TOP", pframe, "TOP", 0,0)

	f.Text = f:CreateFontString(nil, "OVERLAY")
	f.Text:SetFont(L["Fonts\\skurri.ttf"], self.db.profile.titlesize, "OUTLINE")
	f.Text:ClearAllPoints()
	f.Text:SetTextColor(cRed,cGreen,cBlue, 1)
	f.Text:SetWidth(200)
	f.Text:SetHeight(25)
	f.Text:SetPoint("CENTER", f, "CENTER")
	f.Text:SetJustifyH("CENTER")
	f.Text:SetJustifyV("MIDDLE")
	if text ~= "nil" then	f.Text:SetText(text) end
	
	f:Hide()

	return f
end
