--<< ====================================================================== >>--
-- Options Setup                                                              --
--<< ====================================================================== >>--

local L = AceLibrary("AceLocale-2.2"):new("Omniscience")
local dewdrop = AceLibrary:HasInstance("Dewdrop-2.0") and AceLibrary("Dewdrop-2.0") or nil
local LSM = LibStub("LibSharedMedia-3.0")
local waterfall = AceLibrary:HasInstance("Waterfall-1.0") and AceLibrary("Waterfall-1.0") or nil
local opts = {}

function Omniscience:RegisterOpts()
      opts = {
	type = "group",
	args = {
		header = {name = "|cFF77BBFF"..L["Omniscience"], type = "header", order = 1},
		group = { 
			name = L["Group"], desc = L["DescGroup"], type = "group", order = 2,
			args = {
				hostile = {
					name = L["Hostile"], desc=L["DescHostile"], type = "group", order = 1,
					args = {
						enabled = {
							name = L["Enabled"], desc=L["DescEnabled"], type = "toggle", order = 1,
							get = function() return self.db.profile.enabled.hostile end,
							set = function(f) self.db.profile.enabled.hostile = f end,},
					},
				},		
				friendly = {
					name = L["Friendly"], desc=L["DescFriendly"], type = "group", order = 2,
					args = {
						enabled = {
							name = L["Enabled"], desc=L["DescEnabled"], type = "toggle", order = 1,
							get = function() return self.db.profile.enabled.friendly end,
							set = function(f) self.db.profile.enabled.friendly = f end,},
					},
				},
			},
		},
		bar = {
			name = L["Bar"], desc = L["DescBar"], type = "group", order = 3,
			args = {
				texture = {
					name = L["Bar Texture"], desc=L["DescBarTexture"], type = "text", order = 2,
					get = function() return self.db.profile.texture end,
					set = function(f) self.db.profile.texture = f end,
					validate = LSM:List("statusbar")},
				scale = {
					name = L["Bar Scale"], desc=L["DescBarScale"], type = "range", order = 3,
					get = function() return self.db.profile.barscale end,
					set = function(f) self.db.profile.barscale = f end, min = 0.5, max = 2.0, step = 0.05},
				textsize = {
					name = L["Text Size"], desc=L["DescTextSize"], type = "range", order = 4,
					get = function() return self.db.profile.textsize end,
					set = function(f) self.db.profile.textsize = f end, min = 8, max=20, step = 1},
				height = {
					name = L["Bar Height"], desc=L["DescBarHeight"], type = "range", order = 5,
					get = function() return self.db.profile.barheight end,
					set = function(f) self.db.profile.barheight = f end, min = 8, max=30, step = 1},
				width = {
					name = L["Bar Width"], desc=L["DescBarWidth"], type = "range", order = 6,
					get = function() return self.db.profile.barwidth end,
					set = function(f) self.db.profile.barwidth = f end, min = 50, max=400, step = 1},
				reverse = {
					name = L["Reverse"], desc=L["DescReverse"], type = "group", order = 7,
					args = {
						hostile = {
							name = L["Hostile"], desc=L["DescHostile"], type = "toggle", order = 1,
							get = function() return self.db.profile.reverse.hostile end,
							set = function(f) self.db.profile.reverse.hostile = f end},
						friendly = {
							name = L["Friendly"], desc=L["DescFriendly"], type = "toggle", order = 2,
							get = function() return self.db.profile.reverse.friendly end,
							set = function(f) self.db.profile.reverse.friendly = f end},
					}
				},
				growup = {
					name = L["Grow Up"], desc=L["DescGrowup"], type = "group", order = 8,
					args = {
						anchor1 = {
							name = L["Anchor"].." 1", desc=L["DescGrowup"], type = "toggle", order = 1,
							get = function() return self.db.profile.growup[1] end,
							set = function(f) self.db.profile.growup[1] = f; if self:IsCandyBarGroupRegistered("Omniscience-1") then self:SetCandyBarGroupGrowth("Omniscience-1", self.db.profile.growup[1]) end  end},
						anchor2 = {
							name = L["Anchor"].." 2", desc=L["DescGrowup"], type = "toggle", order = 2,
							get = function() return self.db.profile.growup[2] end,
							set = function(f) self.db.profile.growup[2] = f; if self:IsCandyBarGroupRegistered("Omniscience-2") then self:SetCandyBarGroupGrowth("Omniscience-2", self.db.profile.growup[3]) end  end},
					}
				},					
			},
		},
		title = {
			name = L["Title"], desc = L["DescTitle"], type = "group", order = 4,
			args = {
				size = {
					name = L["Title Size"], desc=L["DescTitleSize"], type = "range", order = 1,
					get = function() return self.db.profile.titlesize end,
					set = function(f) self.db.profile.titlesize = f; for i=1, 3 do self.titles[i].Text:SetFont(L["Fonts\\skurri.ttf"], f, "OUTLINE") end end,
					min = 8, max = 20, step = 1},
				title1 = {
					name = L["Title"].." 1", desc = L["DescTitleNum"].." 1.", type = "group", order = 2,
					args = {
						text = {
							name = L["Title Text"], desc=L["DescTitleText"], type = "text", order = 1,
							get = function() return self.db.profile.titletext[1] end,
							set = function(f) self.db.profile.titletext[1] = f; if f == "nil" then f = "" end; self.titles[1].Text:SetText(f) end,usage="<text> - nil to clear"},
						color = {
							name = L["Title Color"], desc=L["DescTitleColor"], type = "color", order = 2,
							get = function() return self.db.profile.titlecolors[1][1], self.db.profile.titlecolors[1][2], self.db.profile.titlecolors[1][3] end,
							set = function(r,g,b) self.db.profile.titlecolors[1] = {r,g,b}; self.titles[1].Text:SetTextColor(r,g,b) end,},
					},
				},
				title2 = {
					name = L["Title"].." 2", desc = L["DescTitleNum"].." 2.", type = "group", order = 3,
					args = {
						text = {
							name = L["Title Text"], desc=L["DescTitleText"], type = "text", order = 1,
							get = function() return self.db.profile.titletext[2] end,
							set = function(f) self.db.profile.titletext[2] = f; if f == "nil" then f = "" end; self.titles[2].Text:SetText(f) end,usage="<text> - nil to clear"},
						color = {
							name = L["Title Color"], desc=L["DescTitleColor"], type = "color", order = 2,
							get = function() return self.db.profile.titlecolors[2][1], self.db.profile.titlecolors[2][2], self.db.profile.titlecolors[2][3] end,
							set = function(r,g,b) self.db.profile.titlecolors[2] = {r,g,b}; self.titles[2].Text:SetTextColor(r,g,b)  end,},
					},
				},
			},
		},
		spacer_toggles = {name = " ", type = "header", order = 6},
		targetonly = {
			name = L["Target Only"], desc=L["DescTargetOnly"], type = "toggle", order = 2, 
			get = function() return self.db.profile.targetonly end,
			set = function(f) self.db.profile.targetonly = f end,},

		spacer_executes = {name = " ", type = "header", order = 12},
		test = {name = L["Test"], desc=L["DescTest"], type = "execute", func = function() self:RunTest() end, order = 13},
		lock = {name = L["Lock"], desc=L["DescLock"], type = "execute", func = function() self:ToggleAnchors() end, order = 14},
		stop = {name = L["Stop"], desc=L["DescStop"], type = "execute", func = function() self:KillAllBars() end, order = 15},
		dropdown = {
			name = L["Dropdown"], desc=L["DescDropdown"], type = "execute", guiHidden = true, order = 16,
			func =  function()
				dewdrop:Open(UIParent, 'children', function() dewdrop:FeedAceOptionsTable(opts) end,'cursorX', true, 'cursorY', true)
			end,
			hidden = function() return not dewdrop end,
		},
		gui = {
			name = L["GUI"], desc=L["DescGUI"], type = "execute", guiHidden = true, order = 17,
			func =  function()
				waterfall:Open("Omniscience")
			end,
			hidden = function() return not waterfall end,
		},

	},
}

	if waterfall then
		waterfall:Register(
			"Omniscience",
			"aceOptions", opts,
			"colorR", .6, "colorG", .5, "colorB", .8
		)
	end

	self:RegisterChatCommand({"/omniscience"}, opts)
end
